
from liteai.core import Message
from liteai.api import chat
from liteai.utils import set_logger, show_response
from loguru import logger

from tests.base import BasicTestCase


class TestDoubao(BasicTestCase):
    @classmethod
    def setUpClass(cls):
        set_logger(__name__)
        logger.info("start doubao test job")

    def basic_llm(self):
        super().basic_llm("doubao-lite-4k")



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@Time    :   2024/06/25 18:39:23
@Author  :   ChenHao
@Description  :   
@Contact :   jerrychen1990@gmail.com
'''

from dotenv import load_dotenv
from liteai.config import LITEAI_ENV, LOG_HOME
from snippets import set_logger

load_dotenv()
set_logger(LITEAI_ENV, __name__, log_dir=LOG_HOME)



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@Time    :   2024/08/12 19:54:53
@Author  :   ChenHao
@Description  :   工具类
@Contact :   jerrychen1990@gmail.com
'''


from typing import Any, Callable

from litellm import Field
from loguru import logger
from openai import BaseModel

from liteai.core import Parameter, ToolDesc


class Tool(BaseModel):
    tool_desc: ToolDesc = Field(..., description="工具描述")
    callable: Callable = Field(..., description="工具执行函数")

    def execute(self, *args, **kwargs) -> Any:
        return self.callable(*args, **kwargs)


def get_current_context(time_type: str):
    import datetime
    import time
    fmt = '%Y-%m-%d %H:%M:%S' if time_type == 'datetime' else '%Y-%m-%d'
    current_time = datetime.datetime.fromtimestamp(time.time()).strftime(fmt)
    return dict(current_time=current_time)


CurrentContextToolDesc = ToolDesc(name="current_context", description="获取当前时间",
                                  parameters=[Parameter(name="time_type", description="时间格式，分为两种datetime:精确到秒, date:精确到日",
                                                        type="string", required=True)],
                                  content_resp="正在帮您获取当前时间...")

CURRENT_CONTEXT_TOOL = Tool(tool_desc=CurrentContextToolDesc, callable=get_current_context)

if __name__ == "__main__":
    resp = CURRENT_CONTEXT_TOOL.execute(time_type="datetime")
    logger.info(resp)



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@Time    :   2024/08/08 17:19:44
@Author  :   ChenHao
@Description  :   测试公共函数
@Contact :   jerrychen1990@gmail.com
'''
from liteai.api import list_models
import unittest

from loguru import logger

from liteai.utils import set_logger


class TestCommon(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        set_logger(__name__)
        logger.info("start test common api")

    def test_list_model(self):
        models = list_models()
        logger.info(f"models: {models}")



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@Time    :   2024/06/25 16:53:45
@Author  :   ChenHao
@Description  :
@Contact :   jerrychen1990@gmail.com
'''

from typing import List
from liteai.core import ModelCard
from liteai.provider.doubao import DoubaoProvider
from liteai.provider.minimax import MinimaxProvider
from liteai.provider.tgi import TGIProvider
from liteai.provider.zhipu import ZhipuProvider
from liteai.provider.qwen import QwenProvider
from liteai.provider.open_ai import OpenAIProvider
from liteai.provider.lite_llm import LiteLLMProvider


from liteai.provider.base import BaseProvider
from dotenv import load_dotenv
load_dotenv()  # 默认会加载当前目录下的 .env 文件


_ALL_PROVIDERS: List[BaseProvider] = [ZhipuProvider, QwenProvider, OpenAIProvider, DoubaoProvider, MinimaxProvider, LiteLLMProvider, TGIProvider]
_PROVIDER_MAP = {p.key: p for p in _ALL_PROVIDERS}


def get_provider_key(model_name: str) -> str:
    model_name = model_name.lower()
    if "tgi" in model_name:
        provider_name = TGIProvider.key
    elif "glm" in model_name:
        provider_name = ZhipuProvider.key
    elif model_name in ["emohaa"]:
        provider_name = ZhipuProvider.key
    elif "qwen" in model_name:
        provider_name = QwenProvider.key
    elif "gpt" in model_name:
        provider_name = OpenAIProvider.key
    elif "doubao" in model_name:
        provider_name = DoubaoProvider.key
    elif "abab" in model_name or "speech-01" in model_name:
        provider_name = MinimaxProvider.key
    else:
        raise ValueError(f"Unsupported model: {model_name}")
    return provider_name


def get_provider(model: ModelCard, api_key: str = None, base_url: str = None, **kwargs):
    provider_name = model.provider
    if provider_name not in _PROVIDER_MAP:
        raise ValueError(f"Unsupported provider: {provider_name}")

    provider_cls = _PROVIDER_MAP[provider_name]
    provider = provider_cls(api_key=api_key, base_url=base_url, **kwargs)
    return provider



import unittest


from liteai.voice import *
from liteai.utils import set_logger
from loguru import logger


class TestVoice(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        set_logger(__name__)
        logger.info("start test voice job")

    def test_file_voice(self):
        voice_path = os.path.join(os.path.abspath(os.path.dirname(__file__)), "hello.mp3")
        voice = file2voice(voice_path)
        play_voice(voice)



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@Time    :   2024/06/25 18:52:00
@Author  :   ChenHao
@Description  :   
@Contact :   jerrychen1990@gmail.com
'''


from typing import Any, List, Tuple

from litellm import completion
from loguru import logger

from liteai.core import ModelResponse, Message, ToolDesc, Usage
from liteai.provider.base import BaseProvider
from snippets.utils import add_callback2gen

from liteai.utils import acc_chunks, get_text_chunk, image2base64


class LiteLLMProvider(BaseProvider):
    key: str = "litellm"
    allow_kwargs = {"stream", "temperature", "top_p", "max_tokens"}
    api_key_env = "LLM_API_KEY"

    def __init__(self, api_key: str = None, base_url: str = None):
        self.base_url = base_url
        super().__init__(api_key=api_key)

    def _support_system(self, model: str):
        return True

    def _get_custom_provider(self, model: str):
        if "tgi" in model:
            return "huggingface"

    def pre_process(self, model: str, messages: List[Message], tools: List[ToolDesc], stream: bool, **kwargs) -> Tuple[List[dict], dict]:
        messages, tools, kwargs = super().pre_process(model, messages, tools, stream, **kwargs)
        for message in messages:
            # logger.debug(f"{message=}")
            if message.get("image"):
                base64 = image2base64(message["image"])
                message["content"] = [dict(type="text", text=message["content"]),
                                      dict(type="image_url", image_url=dict(url="data:image/jpeg;base64," + base64))]
                del message["image"]
        return messages, tools, kwargs

    def _parse_model(self, model: str) -> str:
        if "tgi" in model:
            return "huggingface/" + model

    def _inner_complete_(self, model: str, messages: List[dict], stream: bool, tools, ** kwargs) -> Any:
        model = self._parse_model(model)
        response = completion(
            model=model,
            messages=messages,
            api_base=self.base_url,
            stream=stream,
            **kwargs
        )
        return response

    def post_process(self, response, **kwargs) -> ModelResponse:
        logger.debug(f"{response=}")
        content = response.choices[0].message.content
        usage = Usage(**response.usage.model_dump())
        return ModelResponse(content=content, usage=usage)

    def post_process_stream(self, response, **kwargs) -> ModelResponse:
        gen = (e for e in (get_text_chunk(chunk) for chunk in response) if e)
        gen = add_callback2gen(gen, acc_chunks)
        return ModelResponse(content=gen)


if __name__ == "__main__":
    provider = LiteLLMProvider(base_url="http://36.103.167.117:8101")
    messages = [Message(role="user", content="你好")]
    resp = provider.complete(messages=messages, model="tgi_glm2_12b", stream=False)
    print(resp.content)



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@Time    :   2024/06/25 16:39:05
@Author  :   ChenHao
@Description  :   
@Contact :   jerrychen1990@gmail.com
'''
from typing import Any, List, Tuple

from loguru import logger
from volcenginesdkarkruntime import Ark

from liteai.config import ARK_ENDPOINT_MAP
from liteai.core import ModelCard, ModelResponse, Message, ToolDesc, Usage
from liteai.provider.base import BaseProvider
from snippets import add_callback2gen

from liteai.utils import get_text_chunk, image2base64, acc_chunks


class DoubaoProvider(BaseProvider):
    key: str = "doubao"
    allow_kwargs = {"do_sample", "stream", "temperature", "top_p", "max_tokens"}
    api_key_env = "ARK_API_KEY"

    def __init__(self, api_key: str = None, **kwargs):
        super().__init__(api_key=api_key)
        self.client = Ark(api_key=self.api_key)

    def pre_process(self, model: ModelCard, messages: List[Message], tools: List[ToolDesc], stream: bool, **kwargs) -> Tuple[List[dict], dict]:
        messages, tools, kwargs = super().pre_process(model, messages, tools, stream, **kwargs)
        for message in messages:
            # logger.debug(f"{message=}")
            if message.get("image"):
                base64 = image2base64(message["image"])
                message["content"] = [dict(type="text", text=message["content"]),
                                      dict(type="image_url", image_url=dict(url=base64))]
                del message["image"]
        return messages, tools, kwargs

    def _inner_complete_(self, model: str, messages: List[dict], stream: bool, ** kwargs) -> Any:
        endpoint = ARK_ENDPOINT_MAP[model]
        completion = self.client.chat.completions.create(
            model=endpoint,
            messages=messages,
            stream=stream,
            **kwargs
        )
        logger.debug(f"{completion=}")
        return completion

    def post_process(self, response, **kwargs) -> ModelResponse:
        content = response.choices[0].message.content
        usage = Usage(**response.usage.model_dump())
        return ModelResponse(content=content, usage=usage)

    def post_process_stream(self, response, **kwargs) -> ModelResponse:

        gen = (e for e in (get_text_chunk(chunk) for chunk in response) if e)
        gen = add_callback2gen(gen, acc_chunks)
        return ModelResponse(content=gen)


if __name__ == "__main__":
    provider = DoubaoProvider()
    messages = [Message(role="user", content="你好")]
    resp = provider.complete(messages=messages, model="doubao-lite-4k", stream=False)
    print(resp.content)



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@Time    :   2024/07/31 15:22:33
@Author  :   ChenHao
@Description  : OpenAI测试
@Contact :   jerrychen1990@gmail.com
'''


import unittest
from liteai.core import Message
from liteai.api import chat
from liteai.utils import set_logger, show_response
from loguru import logger

from tests.base import BasicTestCase


class TestOpenAI(BasicTestCase):
    @classmethod
    def setUpClass(cls):
        set_logger(__name__)
        logger.info("start test openai")

    def basic_llm(self):
        super().basic_llm("gpt-3.5-turbo")

    def test_vision_chat(self):
        question = "这张图里有什么?"
        image_path = "./data/Pikachu.png"
        model = "gpt-4o-mini"
        messages = [Message(role="user", content=question, image=image_path)]
        resp = chat(model=model, messages=messages, stream=False, temperature=0.)
        show_response(resp)
        self.assertTrue("皮卡丘" in resp.content)

    @unittest.skip("本地模型不一定部署")
    def test_local_model(self):
        question = "作一首五言绝句"
        model = "glm4-9b-local"
        base_url = "http://36.103.167.117:8000/v1/"
        messages = [Message(role="user", content=question)]
        resp = chat(model=model, messages=messages, provider="openai", stream=False, temperature=0.6, base_url=base_url)
        show_response(resp)



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@Time    :   2024/07/31 15:50:42
@Author  :   ChenHao
@Description  :   
@Contact :   jerrychen1990@gmail.com
'''

from enum import Enum
from typing import Any, Dict, Iterable, List, Optional
from pydantic import BaseModel, Field
from typing import Optional
import warnings

warnings.filterwarnings("ignore", message=".*protected_namespaces.*")


class BaseModel(BaseModel):
    class MyConfig:
        protected_namespaces = ()


class Message(BaseModel):
    role: str = Field(description="The role of the sender in the conversation.", pattern="^(system|user|assistant)$", example="user")
    content: str | Iterable[str] = Field(description="The content of the message.")
    image: Optional[str] = Field(description="local path of image", default=None)


class Usage(BaseModel):
    prompt_tokens: Optional[int] = Field(description="输入token数量", default=None)
    completion_tokens: Optional[int] = Field(description="输出token数量", default=None)
    total_tokens: Optional[int] = Field(description="输入输出token数量总和", default=None)

    @property
    def total_tokens(self):
        return self.total_tokens if self.total_tokens is not None else self.prompt_tokens + self.completion_tokens


class LLMGenConfig(BaseModel):
    temperature: float = Field(description="temperature", default=0.7)
    max_tokens: Optional[int] = Field(description="max_tokens", default=None)


class Parameter(BaseModel):
    name: str = Field(description="参数名称")
    type: str = Field(description="参数类型")
    description: str = Field(description="参数描述")
    required: bool = Field(description="是否必填")


class ToolDesc(BaseModel):

    name: str = Field(..., description="工具名称")
    description: str = Field(..., description="工具描述")
    parameters: List[Parameter] = Field(..., description="工具参数")
    content_resp: Optional[str] = Field(description="调用工具时，返回的文字内容", default=None)

    def to_markdown(self):
        tool_info = f"**[名称]**:{self.name}\n\n**[描述]**:{self.description}\n\n"
        param_infos = []
        for parameter in self.parameters:
            param_infos.append(
                f"- **[参数名]**:{parameter.name}\n\n **[类型]**:{parameter.type}\n\n **[描述]**:{parameter.description}\n\n **[必填]**:{parameter.required}")
        param_infos = "\n\n".join(param_infos)
        return tool_info + param_infos


class ToolCall(BaseModel):
    tool_call_id: str = Field(description="工具调用ID,用于跟踪调用链")
    name: str = Field(description="工具名称")
    parameters: Dict[str, Any] = Field(description="工具调用的参数")
    extra_info: dict = Field(description="额外的信息", default=dict())
    resp: Any = Field(description="工具执行的返回结果,为执行时为None", default=None)

    def to_markdown(self):
        return f"**[调用工具]**: {self.name} **[参数]**: {self.parameters} **[返回结果]**: {self.resp}"

    def to_assistant_message(self):
        return f"调用工具: {self.name}, 参数: {self.parameters}, 返回结果: {self.resp}"


class ModelResponse(BaseModel):
    content: Optional[str | Iterable[str]] = Field(description="模型的回复，字符串或者生成器", default=None)
    image: Optional[str] = Field(description="图片URL", default=None)
    # tool_calls: Optional[list[ToolCall]] = Field(description="工具调用列表", default=list())
    usage: Optional[Usage] = Field(description="token使用情况", default=None)
    # perf: Optional[Perf] = Field(description="性能指标", default=None)
    tool_calls: Optional[List[ToolCall]] = Field(description="工具调用列表", default=[])
    details: Optional[dict] = Field(description="请求模型的细节信息", default=dict())

    class Config:
        protected_namespaces = ()


class Voice(BaseModel):
    byte_stream: Iterable[bytes] | bytes = Field(description="音频字节流")
    file_path: Optional[str] = Field(description="文件路径", default=None)


class ModelType(str, Enum):
    LLM = "llm"
    TTS = "tts"
    EMBEDDING = "embedding"


class ModelCard(BaseModel):
    name: str = Field(description="模型名称")
    model_type: ModelType = Field(description="模型类型", default=ModelType.LLM)
    description: str = Field(description="模型描述", default="")
    support_system: bool = Field(description="是否支持系统对话", default=True)
    support_vision: bool = Field(description="是否支持图像输入", default=False)
    provider: str = Field(description="模型调用器key", default=None)





from liteai.core import Message
from liteai.api import chat, embedding
from liteai.tool import CurrentContextToolDesc
from liteai.utils import get_embd_similarity, set_logger, show_embeds, show_response
from loguru import logger

from tests.base import BasicTestCase


class TestZhipu(BasicTestCase):
    @classmethod
    def setUpClass(cls):
        set_logger(__name__)
        logger.info("start test job")

    def basic_llm(self):
        super().basic_llm("glm-4-air")

    def test_vision_chat(self):
        question = "这张图里有什么?"
        image_path = "./data/Pikachu.png"
        model = "glm-4v"
        messages = [Message(role="user", content=question, image=image_path)]
        resp = chat(model=model, messages=messages, stream=False, temperature=0.)
        show_response(resp)
        self.assertTrue("皮卡丘" in resp.content)

    def test_not_support_system_model(self):
        system = "用英文回答我的问题, 80个单词以内"
        question = "列出国土面积最大的五个国家"
        model = "chatglm_12b"
        messages = [Message(role="system", content=system),
                    Message(role="user", content=question)]
        response = chat(model=model, messages=messages, stream=False, temperature=0.)
        show_response(response)

    def test_lingxin_model(self):
        model = "emohaa"
        meta = {
            "user_info": "30岁的男性软件工程师，兴趣包括阅读、徒步和编程",
            "bot_info": "Emohaa是一款基于Hill助人理论的情感支持AI，拥有专业的心理咨询话术能力",
            "bot_name": "Emohaa",
            "user_name": "张三"
        }
        messages = [
            {"role": "system", "content": "你的名字叫Aifori"},
            {
                "role": "assistant",
                "content": "你好，我是Emohaa，很高兴见到你。请问有什么我可以帮忙的吗？"
            },
            {
                "role": "user",
                "content": "最近我感觉压力很大，情绪总是很低落。"
            },
            {
                "role": "assistant",
                "content": "听起来你最近遇到了不少挑战。可以具体说说是什么让你感到压力大吗？"
            },
            {
                "role": "user",
                "content": "主要是工作上的压力，任务太多，总感觉做不完。"
            }
        ]
        resp = chat(model=model, messages=messages, stream=False, meta=meta)
        logger.info(f"{resp.content=}")

    def test_tool_use(self):
        question = "今天是几号了"
        model = "glm-4-0520"
        tools = [CurrentContextToolDesc]
        response = chat(model=model, messages=question, tools=tools, stream=True, temperature=0.)
        content = show_response(response)
        self.assertIsNotNone(response.tool_calls)
        self.assertEquals(CurrentContextToolDesc.content_resp, content)
        self.assertEquals("current_context", response.tool_calls[0].name)

        question = "世界上面积第三大的国家是哪个？"
        model = "glm-4-0520"
        tools = [CurrentContextToolDesc]
        # tools = []
        response = chat(model=model, messages=question, tools=tools, stream=True, temperature=0.)
        self.assertEqual(0, len(response.tool_calls))
        content = show_response(response)
        # logger.info("content:\n"+content)
        self.assertTrue("中国" in content)

    def test_embd(self):
        tests = ["你好", "hello"]
        embds = embedding(texts=tests, model="embedding-3", batch_size=2, dimensions=512)
        show_embeds(embds)
        self.assertEquals(2, len(embds))
        self.assertEquals(512, len(embds[0]))
        sim = get_embd_similarity(*embds)
        logger.info(f"{sim=:.2f}")
        self.assertGreaterEqual(sim, 0.5)



# -*- coding: utf-8 -*-
"""
-------------------------------------------------
   File Name：     setup
   Description :
   Author :       chenhao
   date：          2021/4/6
-------------------------------------------------
   Change Activity:
                   2021/4/6:
-------------------------------------------------
"""
import sys

from setuptools import find_packages, setup

from snippets.utils import get_latest_version, get_next_version, read2list


def get_install_req():
    req = read2list("requirements.txt")
    req = [e for e in req if not "#" in e and e.strip()]
    return req


if __name__ == "__main__":
    name = "liteai"
    if len(sys.argv) >= 4 and sys.argv[-1].startswith("v"):
        version = sys.argv.pop(-1)
    else:
        latest_version = get_latest_version(name)
        version = get_next_version(latest_version)
    print(f"version: {version}")
    install_req = get_install_req()
    print(f"install_req: {install_req}")
    setup(
        name=name,
        version=version,
        install_requires=install_req,
        packages=find_packages(exclude=['tests*']),
        package_dir={"": "."},
        package_data={},
        url='https://github.com/jerrychen1990/LiteAI',
        license='MIT',
        author='Chen Hao',
        author_email='jerrychen1990@gmail.com',
        zip_safe=True,
        description='use ai lite',
        long_description="use ai lite"
    )



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@Time    :   2024/07/31 15:22:33
@Author  :   ChenHao
@Description  : OpenAI测试
@Contact :   jerrychen1990@gmail.com
'''

import os

from liteai.core import Voice
from liteai.api import tts
from liteai.utils import set_logger
from liteai.voice import play_voice
from loguru import logger

from tests.base import BasicTestCase


class TestMiniMax(BasicTestCase):
    @classmethod
    def setUpClass(cls):
        set_logger(__name__)
        logger.info("start test minimax")

    def basic_llm(self):
        super().basic_llm(model="abab6.5s-chat")

    def test_tts(self):
        text = "你好呀，我是liteai"
        tgt_path = os.path.abspath(os.path.dirname(__file__)) + "/tmp/test_minimax_1.mp3"
        voice: Voice = tts(text=text, model="speech-01-turbo", tgt_path=tgt_path, stream=False)
        play_voice(voice)
        os.remove(tgt_path)
        text = "你好呀，我是流式的一段话"
        tgt_path = os.path.abspath(os.path.dirname(__file__)) + "/tmp/test_minimax_2.mp3"
        voice: Voice = tts(text=text, model="speech-01-turbo", tgt_path=tgt_path, stream=True)
        play_voice(voice)
        play_voice(voice)
        os.remove(tgt_path)
